/*
 * Created on 27.06.2004
 *
 * To change the template for this generated file go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
package sqlplus;

import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.sql.SQLException;
import java.sql.Types;

/**
 * @author Dietmar Aust
 * 
 * To change the template for this generated type comment go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
public class OracleExportFileGenerator {

	private static String DEFAULT_DATE_FORMAT = "DD.MM.YYYY HH24:MI:SS";

	private static String SEPCHAR_COMMA = ",";

	private static String SEPCHAR_SEMICOLON = ";";

	private static String SEPCHAR_TAB = "<TAB>";

	private Connection conn = null;
	
	private String getColumnName(ResultSetMetaData meta, int colNo){
		String colName="";
		try {
			colName = "\"" + meta.getColumnName(colNo) + "\"";
		} catch (SQLException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return colName; 
	}

	private void writeSpoolFile(String sqlQuery, ResultSetMetaData meta,
			boolean includeHeadings, boolean maskQuote, String sepChar, String dateFormat,
			String numberFormat, String outputFileName, boolean useStaticFile,
			String spoolFileName) throws SQLException {
		try {
			// open file
			PrintWriter out = new PrintWriter(new FileWriter(outputFileName));
			out.println("set feedback off");
			out.println("set flush off");
			out.println("set heading off");
			out.println("set headsep off");
			out.println("set timing off");
			out.println("set pagesize 0");
			out.println("set linesize 32767");
			out.println("set recsep off");
			out.println("set termout off");
			out.println("set trimspool on");
			out.println("set verify off");
			out.println("set colsep ';'");
			out.println("set wrap off");
			out.println("set pause off");
			out.println("set sqlblanklines on");
			out.println("set arraysize 500");
			out.println();

			if (useStaticFile == false) {
				out.println("column filename new_value FILENAME;");
				out.println(spoolFileName);
				out.println("spool &&FILENAME;");
			} else {
				out.println("spool " + spoolFileName + ";");
			}

			/*
			 * include headings
			 */
			if (includeHeadings == true) {
				for (int i = 1; i <= meta.getColumnCount(); i++) {
					if (i == 1) {
						out.print("SELECT ");
					} else {
						if (sepChar.equals(SEPCHAR_TAB))
							out.print(" || chr(9) || ");
						else
							out.print(" ||'" + sepChar + "'|| ");
					}
					out.println("'\"" + meta.getColumnName(i) + "\"'");
				}
				out.println(" FROM dual");
				out.println("/");
				out.println();
			}

			/*
			 * Select values
			 */
			for (int i = 1; i <= meta.getColumnCount(); i++) {
				if (i == 1) {
					out.print("SELECT ");
				} else {
					if (sepChar.equals(SEPCHAR_TAB))
						out.print(" || chr(9) || ");
					else
						out.print(" ||'" + sepChar + "'|| ");
				}
				switch (meta.getColumnType(i)) {
				case Types.VARCHAR:
				case Types.CHAR:
					if (maskQuote==true){
						out.println("'\"' || " + "replace(" + getColumnName(meta,i)
								+ ", '\"', '\"\"')" + " || '\"'");						
					}else{
						out.println("'\"' || " + getColumnName(meta,i) + " || '\"'");
					}
					break;
				case Types.DATE:
					out.println("'\"' || " + "to_char(" + getColumnName(meta,i)
							+ ", '" + dateFormat + "') || '\"' ");
					//out.println("to_char(" + getColumnName(i) + ", '"
					//		+ dateFormat + "')");
					break;
				case Types.BIGINT:
				case Types.DECIMAL:
				case Types.DOUBLE:
				case Types.FLOAT:
				case Types.INTEGER:
				case Types.NUMERIC:
				case Types.REAL:
					// Number Format gesetzt?
					if (numberFormat.equals("")) {
						// NEIN
						if (sepChar.equals(SEPCHAR_COMMA)) {
							out.println("'\"' || " + getColumnName(meta, i)
									+ " || '\"' ");
						} else {
							out.println(getColumnName(meta, i));
						}

					} else {
						// JA
						out.println("to_char(" + getColumnName(meta,i) + ", '"
								+ numberFormat + "') ");
						//out.println("'\"' || " + "to_char(" +
						// getColumnName(i)
						//	+ ", '" + numberFormat + "') || '\"' ");
					}
					break;
				default:
					out.println(getColumnName(meta,i));
				}

			}

			out.println(" FROM (" + sqlQuery + ")");
			out.println("/");
			out.println("spool off");
			//out.println("exit");

			out.flush();
			out.close();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

	}

	/*
	 * Generate the sqlplus script ...
	 */
	public void generateScript(String user, String pwd, String jdbcUrlPart,
			String sql, boolean includeHeaders, boolean maskQuote, String sepChar,
			String dateFormat, String numberFormat, String scriptFilename,
			boolean useStaticFile, String spoolFilename) throws SQLException {
		//String connectURL = "jdbc:oracle:oci:@" + tns;
		String connectURL = "jdbc:oracle:thin:@" + jdbcUrlPart;
		DriverManager.registerDriver(new oracle.jdbc.OracleDriver());

		//DriverManager.registerDriver(new oracle.jdbc.driver.OracleDriver());
		conn = DriverManager.getConnection(connectURL, user, pwd);

		System.out.println("successfully connected!");

		ResultSet rs = conn.createStatement().executeQuery(sql);
		ResultSetMetaData meta = rs.getMetaData();

		writeSpoolFile(sql, meta, includeHeaders, maskQuote, sepChar, dateFormat,
				numberFormat, scriptFilename, useStaticFile, spoolFilename);

		rs.close();

		conn.close();
		System.out.println("successfully generated!");
	}

	
	public void generateExecuteSQLScript(String executeSQLScriptName, String scriptFilename){
		try {
			// open file
			PrintWriter out = new PrintWriter(new FileWriter(executeSQLScriptName));
			out.println("@\"" + scriptFilename + "\"");
			out.println("exit");
			out.println();

			out.flush();
			out.close();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

	}
	
	public static void main(String[] args) throws SQLException {
		OracleExportFileGenerator gen = new OracleExportFileGenerator();

		String user = "olga";
		String pwd = "olga";
		String tns = "olga";

		String sql = "select user, rownum, sysdate, r.* from ref_codes r";

		gen.generateScript(user, pwd, "localhost:1521:olga", sql, true, false, ",",
				"DD.MM.YYYY", "999G999G999D99", "c:\\temp\\query.sql", true,
				"c:\\temp\\out.csv");
	}
}